package sbc.mgclus;

import java.io.File;
import java.util.HashSet;

import org.apache.log4j.Logger;
import org.jgrapht.WeightedGraph;
import org.jgrapht.graph.DefaultWeightedEdge;

import sbc.mgclus.algorithm.*;
import sbc.mgclus.algorithm.scoring.CommonNeighborsS;
import sbc.mgclus.algorithm.scoring.Scoring;
import sbc.mgclus.algorithm.scoring.ScoringDefault;
import sbc.mgclus.algorithm.scoring.ScoringSeeds;
import sbc.mgclus.data.*;
import sbc.mgclus.parser.*;
import sbc.mgclus.writer.OutputWriterTSV;

import java.text.DecimalFormat;

public class MGclus {
		
	private static Logger logger = Logger.getLogger(MGclus.class);
	
	public static void main (String[] args){
		System.err.println("\nMGclus version 1.0\n");
		//parse command line arguments 
		CmdParser cmd = new CmdParser (args);
		
		Logger.getRootLogger().setLevel(cmd.getVerbose());
				
		//if a seed file was specified read seed file
		HashSet<String> seedList = new HashSet<String>();
		if (!(cmd.getSeedsFile() == null)){
			SeedTSVParser seedParser= new SeedTSVParser();
			seedList = seedParser.parseSeeds(cmd.getSeedsFile());
		}
		//read in graph file
		GraphTSVParser graphParser = new GraphTSVParser();
		WeightedGraph<Cluster, DefaultWeightedEdge> graph = graphParser.parseGraph(cmd.getGraphFile(), cmd.getUseWeights(), seedList);
		//create a new common neighbors object
		CommonNeighborsS commonNeighbors = new CommonNeighborsS(graph, cmd.getUseWeights());
		//create new scoring object
		Scoring scoring;
		if (cmd.getSeedsFile() == null){
			scoring = new ScoringDefault(graph, commonNeighbors);
		}
		else{
			logger.info("\nuse seeds\n");
			scoring = new ScoringSeeds(graph, commonNeighbors);
		}
		//create a new clustering object
		ClusteringKL clustering;
		if (cmd.getDelta() != null)
			clustering = new ClusteringKL(graph, scoring, cmd.getDelta());
		else
			clustering = new ClusteringKL(graph, scoring);
		//clustering
		clustering.performClusteringKL();
		//write cluster's to result file
		File outputFile = cmd.getOutputFile();
		//check if a result file was specified...
		if (outputFile == null){
			String path = (cmd.getGraphFile()).getParent();
			String filename;
			if (path == null)
				filename = "mgclus.out."+(cmd.getGraphFile()).getName();
			else
				filename = path+"/mgclus.out."+(cmd.getGraphFile()).getName();
			if (cmd.getDelta() != null){
				DecimalFormat df = new DecimalFormat("0.00");
				filename = filename+"_D"+(df.format(cmd.getDelta()));//String.format("_D%i", rounded);
			}
			else
				filename = filename+"_D0";
			outputFile = new File(filename);
		}
		OutputWriterTSV outputTSV = new OutputWriterTSV(graph, outputFile);
		outputTSV.outputClusterTSV();
	}
}
