package sbc.mgclus.algorithm.scoring;

import java.util.PriorityQueue;

import org.apache.log4j.Logger;
import org.jgrapht.WeightedGraph;
import org.jgrapht.graph.DefaultWeightedEdge;

import sbc.mgclus.algorithm.ClusteringKL;
import sbc.mgclus.data.Cluster;
import sbc.mgclus.data.ClusterPair;
import sbc.mgclus.data.Vertex;

public class ScoringDefault implements Scoring{
	private WeightedGraph<Cluster, DefaultWeightedEdge> graph;
	private CommonNeighborsS commonNeighbors;
	private Logger logger=Logger.getLogger(ClusteringKL.class);
	
	public ScoringDefault(WeightedGraph<Cluster, DefaultWeightedEdge> g, CommonNeighborsS cNB){
		this.graph = g;
		this.commonNeighbors = cNB;
	}
	
	public double calcMergeGain(Cluster c1, Cluster c2){
		//get the merge gains of the single clusters
		double mgC1 = c1.getClusterEfficiency()/(Math.pow(c1.getClusterSize(), 2));
		double mgC2 = c2.getClusterEfficiency()/(Math.pow(c2.getClusterSize(), 2));
		//double mgC1 = c1.getClusterEfficiency()/(c1.getDegree());
		//double mgC2 = c2.getClusterEfficiency()/(c2.getDegree());
		
		//calculate the efficiency of the two merged clusters
		//for all vertices in the two clusters find the number of common neighbors
		double commonNBscore = calcCNScore(c1, c2);
		double mergedEfficiency = graph.getEdgeWeight(graph.getEdge(c1, c2)) + commonNBscore;
		mergedEfficiency += c1.getClusterEfficiency();
		mergedEfficiency += c2.getClusterEfficiency();
		double mgMerged = mergedEfficiency/(Math.pow(c1.getClusterSize()+c2.getClusterSize(), 2));
		//double mgMerged = mergedEfficiency/(c1.getDegree()+c2.getDegree());
		
		/*//debugging 
		logger.debug(String.format("merge gains for: c%s, c%s", c1.getClusterID(), c2.getClusterID()));
		logger.debug(String.format("\tc%s: cluster size: %s, cluster degree %f", c2.getClusterID(), c2.getClusterSize(), c2.getDegree()));
		//logger.debug(String.format("\tcommonNeighbors Score: %f", commonNBscore));
		//logger.debug(String.format("\tEdge Score: %f", graph.getEdgeWeight(graph.getEdge(c1, c2))));
		logger.debug(String.format("\tc%s, %f, %f", c1.getClusterID(), c1.getClusterEfficiency(), mgC1));
		logger.debug(String.format("\tc%s, %f, %f", c2.getClusterID(), c2.getClusterEfficiency(), mgC2));
		logger.debug(String.format("%f, %f, %f", mergedEfficiency, (Math.pow(c1.getClusterSize()+c2.getClusterSize(), 2)), c1.getDegree()+c2.getDegree()));
		logger.debug(String.format("\tpair, %f, %f", mgMerged, mgMerged2));
		logger.debug(String.format("\tmerge efficiency: %f", 2*mgMerged - (mgC1 + mgC2)));
		logger.debug(String.format("\tmerge efficiency: %f", 2*mgMerged2 - (mgC1 + mgC2)));
		*/
		return 2*mgMerged - (mgC1 + mgC2);
	}

	public PriorityQueue<ClusterPair> getInitialClusterPairs() {
		//debugging 
		//logger.debug(String.format("number of total nodes: %d, number of total edges: %f\n", nrNodesTotal, nrEdgesTotal));
		PriorityQueue<ClusterPair> clustersToMerge = new PriorityQueue<ClusterPair>();
		//instantiate a cluster pair for each edge in the graph
		for (DefaultWeightedEdge edge : graph.edgeSet()){
			Cluster c1 = graph.getEdgeSource(edge);
			Cluster c2 = graph.getEdgeTarget(edge);
			double mergeGain = calcMergeGain(c1,c2);
			if (mergeGain >= 0){
				ClusterPair cPair = new ClusterPair(c1, c2, mergeGain);
				clustersToMerge.add(cPair);
			}
			//debugging
			//double w = graph.getEdgeWeight(edge);
			//logger.debug(String.format("pair added between: c%d, c%d, edge weight %f, merge gain %f\n", c1.getClusterID(), c2.getClusterID(), w, mergeGain));
			//logger.debug(String.format("pair added between: c%d (%s), c%d (%s), edge weight %f, merge gain %f", c1.getClusterID(), (c1.getMembers()).get(0), c2.getClusterID(), (c2.getMembers()).get(0), w, mergeGain));
		}
		return clustersToMerge;
	}

	public double calcCNScore(Cluster c1, Cluster c2) {
		double commonNBscore = 0.0;
		for (Vertex v : c1.getMembers()){
			for (Vertex u : c2.getMembers()){
				commonNBscore += ((Math.sqrt(commonNeighbors.getSharedNeighbors(v, u))));
			}
		}
		return commonNBscore;
	}
	
}
